/*
 * Copyright (c) 2011-2013 GoPivotal, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package reactor.core.action;

/**
 * Component that can be injected with {@link Action}s and consume flush events for releasing buffer owned by the
 * pipeline
 *
 * @author Stephane Maldini
 * @author Jon Brisbin
 * @since 1.1
 */
public interface Pipeline<T> extends Flushable<T>{

	/**
	 * Consume events with the passed {@code Action}
	 *
	 * @param action
	 * 		the action listening for values
	 */
	Pipeline<T> add(Action<T> action);


	/**
	 * Consume flush with the passed {@link Flushable}
	 *
	 * @param action
	 * 		the action listening for flush
	 *
	 * @return {@literal this}
	 */
	Pipeline<T> consumeFlush(Flushable<?> action);
}
